<?php

declare(strict_types=1);

namespace Drupal\ragapay\Client;

use GuzzleHttp\Client;
use Psr\Log\LoggerInterface;
use GuzzleHttp\RequestOptions;
use Drupal\Component\Serialization\Json;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\ResponseFields;

/**
 * RagaPay client.
 */
class RagaPayClient {
  public const SESSION_ENDPOINT = '/api/v1/session';
  public const BASE_URL = 'https://checkout.ragapay.com';

  /**
   * Constructs a new RagaPayClient object.
   */
  public function __construct(private readonly Client $httpClient, private readonly LoggerInterface $logger) {
  }

  /**
   * Initializes a new session.
   */
  public function initSession(array $payload): ?array {
    $data = [
      RequestOptions::BODY => Json::encode($payload),
      RequestOptions::HEADERS => [
        'Content-Type' => 'application/json',
      ],
      RequestOptions::ALLOW_REDIRECTS => TRUE,
      RequestOptions::HTTP_ERRORS => FALSE,
    ];
    $url = sprintf('%s%s', self::BASE_URL, self::SESSION_ENDPOINT);
    $response = $this->doRequest($url, 'POST', $data);
    if (!empty($response[ResponseFields::REDIRECT_URL->value])) {
      return $response;
    }
    return NULL;
  }

  /**
   * Does a request.
   */
  protected function doRequest(string $url, string $method, array $data = []): ?array {
    try {
      $response = $this->httpClient->request($method, $url, $data);
      $content = $response->getBody()->getContents();
      $decoded = Json::decode($content);
      if (!empty($decoded[ResponseFields::ERROR_MESSAGE->value])) {
        $this->logger->error($decoded[ResponseFields::ERROR_MESSAGE->value]);
        return [];
      }
      return $decoded;
    }
    catch (\Throwable $throwable) {
      $this->logger->error($throwable->getMessage());
      return [];
    }

    return [];
  }

}
