<?php

declare(strict_types=1);

namespace Drupal\ragapay\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\ResponseFields;
use Drupal\ragapay\Service\RagaPayManager;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Returns responses for RagaPay routes.
 */
final class NotificationController extends ControllerBase {

  /**
   * The controller constructor.
   */
  public function __construct(
    private readonly RagaPayManager $ragaPayManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('ragapay.manager'),
    );
  }

  /**
   * Builds the response.
   */
  public function __invoke(Request $request): JsonResponse {
    $status = FALSE;
    $responseStr = $request->getContent();
    parse_str($responseStr, $notificationData);
    if (!empty($notificationData[ResponseFields::ORDER_NUMBER->value])) {
      $status = $this->ragaPayManager->updateOrder($notificationData[ResponseFields::ORDER_NUMBER->value], $notificationData);
    }
    return new JsonResponse(['status' => $status]);
  }

}
