<?php

declare(strict_types=1);

namespace Drupal\ragapay\Plugin\Commerce\PaymentGateway;

use Drupal\Core\Url;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\commerce_payment\Attribute\CommercePaymentGateway;
use Drupal\ragapay\PluginForm\OffsiteRedirect\RagaPayOffsiteForm;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;

/**
 * Provides the Off-site RagaPay payment gateway.
 */
#[CommercePaymentGateway(
  id: "ragapay_offsite",
  label: new TranslatableMarkup("RagaPay (Off-site redirect)"),
  display_label: new TranslatableMarkup("RagaPay"),
  forms: [
    "offsite-payment" => RagaPayOffsiteForm::class,
  ],
  requires_billing_information: FALSE,
)]
class RagaPayOffsite extends OffsitePaymentGatewayBase {

  public const PASSWORD_CONFIG = 'password';
  public const MERCHANT_CONFIG = 'merchant_key';
  public const NOTIFICATION_CONFIG = 'notification_url';

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form[self::PASSWORD_CONFIG] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#default_value' => $this->configuration[self::PASSWORD_CONFIG] ?? '',
      '#required' => TRUE,
    ];
    $form[self::MERCHANT_CONFIG] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant Key'),
      '#default_value' => $this->configuration[self::MERCHANT_CONFIG] ?? '',
      '#required' => TRUE,
    ];
    $form[self::NOTIFICATION_CONFIG] = [
      '#type' => 'textfield',
      '#title' => $this->t('Notification URL'),
      '#default_value' => Url::fromRoute('ragapay.notification')->setAbsolute()->toString(),
      '#description' => $this->t('Set this notification URL on your RagaPay account.'),
      '#required' => TRUE,
      '#disabled' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration[self::MERCHANT_CONFIG] = $values[self::MERCHANT_CONFIG];
      $this->configuration[self::PASSWORD_CONFIG] = $values[self::PASSWORD_CONFIG];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $payment_storage->create([
      'state' => 'new',
      'amount' => $order->getBalance(),
      'payment_gateway' => $this->parentEntity->id(),
      'order_id' => $order->id(),
    ]);
    $payment->save();
  }

}
