<?php

declare(strict_types=1);

namespace Drupal\ragapay\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\PaymentStatus;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\ResponseFields;

/**
 * Ragapay manager.
 */
final class RagaPayManager {

  /**
   * Constructs a RagaPayManager object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * Updates the order.
   */
  public function updateOrder(string $orderId, array $data): bool {
    $storage = $this->entityTypeManager->getStorage('commerce_order');
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $storage->load($orderId);
    if (!$order) {
      return FALSE;
    }
    $paymentStorage = $this->entityTypeManager->getStorage('commerce_payment');
    $payments = $paymentStorage->loadByProperties(['order_id' => $order->id(), 'payment_gateway' => 'ragapay']);
    if (empty($payments)) {
      return FALSE;
    }
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = reset($payments);
    $payment->setRemoteId($data[ResponseFields::REMOTE_ID->value]);
    $ragaPayStatus = PaymentStatus::from($data[ResponseFields::STATUS->value]);
    $paymentState = $ragaPayStatus->toPaymentState();
    $payment->setState($paymentState);
    $payment->save();

    return TRUE;
  }

}
